#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
from pprint import pprint
import click
from .utils import copy_templates, parse_setup, get_console_scripts


@click.group()
def cli():
    '''Provides transparent scripts to wrap python cli tools.'''


@cli.command()
@click.option('--entry_point', default=None)
@click.option('--all_entry_points', is_flag=True, default=False)
@click.option('--auto_write', is_flag=True, default=True)
@click.option('--scripts_path', default='bin')
def add(entry_point, all_entry_points, auto_write, scripts_path):
    '''Add Scrim scripts for a python project'''

    click.echo()
    if not os.path.exists('setup.py'):
        raise click.UsageError('No setup.py file exists.')

    setup_data = parse_setup('setup.py')
    console_scripts = get_console_scripts(setup_data)

    scripts = []
    if all_entry_points and console_scripts:
        for entry in console_scripts:
            click.echo('Found entry_point: ' + entry)
            more_scripts = copy_templates(entry, auto_write, scripts_path)
            for script in more_scripts:
                click.echo('    Created ' + script)
            scripts.extend(more_scripts)

    elif entry_point:
        if not entry_point in console_scripts:
            click.echo(entry_point + ' not found in your setups entry_points')
            click.echo('You will need to add it afterward if you continue...\n')
            click.confirm('Do you want to continue?', abort=True)
        click.echo('\nCreating scripts for: ' + entry_point)
        more_scripts = copy_templates(entry_point, auto_write, scripts_path)
        for script in more_scripts:
            click.echo('    Created ' + script)
        scripts.extend(more_scripts)

    click.echo('\n\nAdd the following section to your package setup:\n')
    click.echo('scripts=[')
    for script in scripts:
        click.echo("    '{}',".format(script))
    click.echo('],')


@cli.command()
def print_setup():
    setup_data = parse_setup('setup.py')
    pprint(setup_data)

if __name__ == '__main__':
    cli()



# -*- coding: utf-8 -*-
__all__ = ['SHELLS', 'SCRIM_SHELL', 'SCRIM_PATH', 'SCRIM_AUTO_WRITE', 'SCRIM_SCRIPT']
import os
from .utils import get_parent_process

SHELLS = [
    'powershell.exe',
    'cmd.exe'
]
SCRIM_SHELL = os.environ.get('SCRIM_SHELL', None)
SCRIM_PATH = os.environ.get('SCRIM_PATH', None)
SCRIM_AUTO_WRITE = int(os.environ.get('SCRIM_AUTO_WRITE', False))
SCRIM_SCRIPT = os.environ.get('SCRIM_SCRIPT', None)



__all__ = ['Scrim', 'get_scrim']
import os
import atexit
from .globals import SCRIM_AUTO_WRITE, SCRIM_PATH, SCRIM_SHELL, SCRIM_SCRIPT


def init_value(value=None, default=None):
    '''Returns default if value is None'''
    if value is None:
        return default
    return value


class Scrim(list):
    '''The Scrim object is a subclass of list. Append commands to a Scrim that
    you would like to be executed by the Scrim Script that invoked the current
    python process. It helps to think of a Scrim as an interface to the shell
    process where your python code was executed.

    The following environment variables set by Scrim Scripts provide the
    defaults for the Scrim::

    .. envvar:: SCRIM_SHELL

        Name of shell process(powershell.exe, cmd.exe..)
        If this envvar is missing we walk up the process tree until we find a
        compatible shell process.
        Currently only powershell.exe and cmd.exe are implemented

    .. envvar:: SCRIM_PATH

        Path to script file to be written to and then executed after exit

    .. envvar:: SCRIM_AUTO_WRITE

        If True automatically write out the scrim file on program exit

    .. envvar:: SCRIM_SCRIPT

        Path to the Scrim Script that invoked the current python process

    These environment variables should be set by the Scrim shell script, not by
    the user. For testing purposes these values can be overriden by passing in
    any of the following parameters.

    :param shell: Name of the shell process or :envvar:`SCRIM_SHELL`
    :param path: Path to temporary scrim file or :envvar:`SCRIM_PATH`
    :param auto_write: Write scrim file on program exit or :envvar:`SCRIM_AUTO_WRITE`
    :param script: Path to scrim script or :envvar:`SCRIM_AUTO_WRITE`

    .. usage::
        >>> scrimt = Scrim()
        >>> if scrim.shell == 'powershell.exe':
        ...     scrim.append('Write-Host $env:PATH')
        ...
        >>> scrim.write()
    '''

    def __init__(self, path=None, auto_write=None, shell=None, script=None):
        self.shell = init_value(shell, SCRIM_SHELL)
        self.path = init_value(path, SCRIM_PATH)
        self.auto_write = init_value(auto_write, SCRIM_AUTO_WRITE)
        self.script = init_value(script, SCRIM_SCRIPT)
        atexit.register(self.on_exit)

    def __repr__(self):
        args = (self.path, self.auto_write, self.shell, self.script)
        return '{}({}, {}, {}, {})'.format(self.__class__.__name__, *args)

    def to_string(self):
        '''Join this Scrims commands'''

        return '\n'.join(self)

    def write(self):
        '''Write this Scrims commands to its path'''

        if self.path is None:
            raise Exception('Scrims path is None')

        dirname = os.path.dirname(os.path.abspath(self.path))
        if not os.path.exists(dirname):
            try:
                os.makedirs(dirname)
            except:
                raise OSError('Failed to create root for scrim output.')

        with open(self.path, 'w') as f:
            f.write(self.to_string())

    def on_exit(self):
        if not all([self.auto_write, self, self.script, self.path]):
            return

        self.write()


def get_scrim(path=None, auto_write=None, shell=None, script=None, cache={}):
    '''Usual method of getting a :class:`Scrim` instance. Each instance is
    cached so if you call get_scrim again with the same arguments you get the
    same instance.

    .. seealso:: :class:`Scrim`
    '''

    args = (path, auto_write, shell, script)
    if args not in cache:
        cache[args] = Scrim(*args)
    return cache[args]



import re
import sys
import shutil
from subprocess import check_call
from setuptools import setup, find_packages


if sys.argv[-1] == 'cheeseit!':
    check_call('nosetests -v')
    check_call('python setup.py sdist bdist_wheel')
    check_call('twine upload dist/*')
    shutil.rmtree('dist')
    sys.exit()
elif sys.argv[-1] == 'testit!':
    check_call('nosetests -v')
    check_call('python setup.py sdist bdist_wheel upload -r pypitest')
    sys.exit()


def get_info(pyfile):
    '''Retrieve dunder values from a pyfile'''
    info = {}
    info_re = re.compile(r"^__(\w+)__ = ['\"](.*)['\"]")
    with open(pyfile, 'r') as f:
        for line in f.readlines():
            match = info_re.search(line)
            if match:
                info[match.group(1)] = match.group(2)
    return info

info = get_info('scrim/__init__.py')

with open('README.rst', 'r') as f:
    long_description = f.read()


setup(
    name=info['title'],
    version=info['version'],
    url=info['url'],
    license=info['license'],
    author=info['author'],
    author_email=info['email'],
    description=info['description'],
    long_description=long_description,
    install_requires=[
        'click>=6.7',
        'psutil>=5.2'
    ],
    packages=['scrim'],
    package_data={
        'scrim': ['bin/*.*']
    },
    entry_points={
        'console_scripts': [
            'scrim = scrim.__main__:cli'
        ]
    },
    classifiers=(
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ),
)
